import 'dart:io';

import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_upload/src/repository/upload.dart';
import 'package:path_provider/path_provider.dart';
import 'package:rxdart/subjects.dart';
import 'package:uuid/uuid.dart';
import 'package:video_thumbnail/video_thumbnail.dart';

class UploadFileType {
  static const String TEXT = "text";
  static const String IMAGE = "image";
  static const String FILE = "file";
  static const String VIDEO = "video";
  static const String AUDIO = "audio";
}

class UploadState {
  final List<UploadFile> totalInProgress;
  final List<UploadResult> totalDone;

  UploadState(this.totalInProgress, this.totalDone);

  UploadState.named({this.totalInProgress, this.totalDone});

  UploadState copyWith({List<UploadFile> inProgress, List<UploadResult> done}) {
    return UploadState.named(
        totalDone: done ?? this.totalDone,
        totalInProgress: inProgress ?? this.totalInProgress);
  }

  List<UploadFile> get inProgress => totalInProgress;

  List<UploadResult> get done => totalDone;
}

abstract class UploadEvent {}

class UploadStartEvent extends UploadEvent {
  final UploadFile file;

  UploadStartEvent(this.file);
}

class UploadDoneEvent extends UploadEvent {
  final UploadFile file;

  UploadDoneEvent(this.file);
}

class UploadBloc {
  static BehaviorSubject<UploadState> _subject;
  static BehaviorSubject<UploadFile> _inProgressSubject;
  static BehaviorSubject<UploadResult> _doneSubject;
  static UploadState _state;
  final UploadRepository uploadRepository;

  UploadBloc(this.uploadRepository);

  void open() {
    if (_subject == null) {
      _subject = BehaviorSubject<UploadState>();
      _inProgressSubject = BehaviorSubject<UploadFile>();
      _doneSubject = BehaviorSubject<UploadResult>();
      _state = initialState;
      listenToProgress().listen((event) {});
      listenToDone().listen((event) {});
    }
  }

  Stream<void> listenToProgress() async* {
    await for (var uploadFile in _inProgressSubject.asBroadcastStream()) {
      var inProgress = _state.totalInProgress.toList()..add(uploadFile);
      _state = _state.copyWith(inProgress: inProgress);
      _subject.add(_state);
    }
  }

  Stream<void> listenToDone() async* {
    await for (var uploadResult in _doneSubject.asBroadcastStream()) {
      var done = _state.totalDone.toList()..add(uploadResult);
      if (done.length == _state.inProgress.length) {
        clear();
      } else {
        _state = _state.copyWith(done: done);
        _subject.add(_state);
      }
    }
  }

  UploadState get initialState =>
      UploadState.named(totalDone: [], totalInProgress: []);

  Stream<UploadState> get state => _subject?.stream;

  Future<AppResponse<List<Basic>>> uploadFiles(List<File> files, String type,
      {Function(Basic basic) onUploaded}) async {
    try {
      List<Basic> result = [];
      List<UploadFile> uploads = files
          .map((e) => UploadFile.named(id: Uuid().v4(), file: e, type: type))
          .toList();
      uploads.forEach((element) {
        _addInProgress(element);
      });
      for (var upload in uploads) {
        var response = await _uploadFile(upload);
        if (response.isSuccess) {
          if (onUploaded != null) {
            onUploaded(response.data);
          }
          result.add(response.data);
        }
        // // DEBUG
        // await Future.delayed(Duration(seconds: 1));
        // _addDone(UploadResult.named(
        //     file: upload.file,
        //     contentJson: ImageJson.named(
        //         imageUrl:
        //             "https://images.pexels.com/photos/4716252/pexels-photo-4716252.jpeg?auto=compress&cs=tinysrgb&dpr=2&w=500"),
        //     id: upload.id));
        _addDone(UploadResult.named(
            file: upload.file, contentJson: response.data, id: upload.id));
      }
      return AppResponse.named(data: result);
    } catch (e) {
      clear();
      return AppResponse.named(error: "$e");
    }
  }

  void _addInProgress(UploadFile uploadFile) {
    _inProgressSubject.add(uploadFile);
  }

  void _addDone(UploadResult uploadResult) {
    _doneSubject.add(uploadResult);
  }

  Future<AppResponse<Basic>> _uploadFile(UploadFile uploadFile) async {
    switch (uploadFile.type) {
      case UploadFileType.IMAGE:
        return await _uploadImage(uploadFile.file);
        break;
      case UploadFileType.VIDEO:
        return await _uploadVideo(uploadFile.file);
        break;
      // case UploadFileType.AUDIO:
      //   return await _uploadAudio(uploadFile.file);
      //   break;
      default:
        return await _uploadCustomFile(uploadFile.file);
    }
  }

  ///
  /// Image Uplopad
  ///

  Future<AppResponse<ImageData>> _uploadImage(File file) async {
    var response = await uploadRepository.storeFile(file, path: Uuid().v4());
    if (response.isSuccess) {
      return AppResponse.named(data: ImageData.named(url: response.data));
    }
    return AppResponse.named(error: response.error);
  }

  ///
  /// Video Uplopad
  ///

  Future<AppResponse<VideoData>> _uploadVideo(File file) async {
    var thumbnailResponse = await createThumbnail(file.path);
    if (thumbnailResponse.isSuccess) {
      String thumbnail = thumbnailResponse.data;
      String uploadedVideo, uploadedThumbnail;
      try {
        await Future.wait([
          Future(() async {
            AppResponse<String> response = await uploadRepository
                .storeFile(file, path: "videos/" + Uuid().v4());
            if (response.isSuccess) {
              uploadedVideo = response.data;
            }
          }),
          Future(() async {
            AppResponse<String> response = await uploadRepository
                .storeFile(File(thumbnail), path: "thumnbnail/" + Uuid().v4());
            if (response.isSuccess) {
              uploadedThumbnail = response.data;
            }
          }),
        ]);
        if (uploadedVideo != null) {
          return AppResponse.named(
              data: VideoData.named(
                  url: uploadedVideo, thumbnail: uploadedThumbnail));
        }
        return AppResponse.named(error: "Video not uploaded");
      } catch (e) {
        return AppResponse.named(error: "${e.toString()}");
      }
    }
    return AppResponse.named(error: thumbnailResponse.error);
  }

  Future<AppResponse<String>> createThumbnail(String videoPath) async {
    try {
      String path = (await getTemporaryDirectory()).path;
      var thumbnail = await VideoThumbnail.thumbnailFile(
          video: videoPath,
          imageFormat: ImageFormat.JPEG,
          thumbnailPath: path,
          quality: 50);
      return AppResponse.named(data: thumbnail);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  ///
  /// File Upload
  ///
  Future<AppResponse<Basic>> _uploadCustomFile(File file) async {
    var response = await uploadRepository.storeFile(file, path: Uuid().v4());
    if (response.isSuccess) {
      return AppResponse.named(
          data: FileData.named(
        url: response.data,
      ));
    }
    return AppResponse.named(error: response.error);
  }

  ///
  /// Audio Upload
  ///
  // Future<AppResponse<AudioJson>> _uploadAudio(File file) async {
  //   var response = await UploadController.uploadFile(file, path: Uuid().v4());
  //   if (response.isSuccess) {
  //     AudioJson audioJson = AudioJson.named(
  //         audioName: "Rec-${DateTime.now().millisecondsSinceEpoch}",
  //         audioUrl: response.data);
  //     return AppResponse.named(data: audioJson);
  //   }
  //   return AppResponse.named(error: response.error);
  // }

  void clear() {
    _state = initialState;
    _subject.add(_state);
  }

  void close() {
    if (_subject.isBroadcast) {
      _subject.close();
      _inProgressSubject.close();
      _doneSubject.close();
      _subject = null;
      _inProgressSubject = null;
      _doneSubject = null;
    }
  }
}

class UploadFile {
  final String id;
  final File file;
  final String url;
  final String type;

  UploadFile(this.id, this.file, this.url, this.type);

  UploadFile.named({this.id, this.file, this.url, this.type});

  UploadFile copyWith({String id, File file, String url, String type}) {
    return UploadFile.named(
        id: id ?? this.id,
        file: file ?? this.file,
        url: url ?? this.url,
        type: type ?? this.type);
  }
}

class UploadResult {
  final String id;
  final File file;
  final Basic contentJson;

  UploadResult(this.id, this.file, this.contentJson);

  UploadResult.named({this.id, this.file, this.contentJson});

  UploadResult copyWith({String id, File file, Basic contentJson}) {
    return UploadResult.named(
        id: id ?? this.id,
        file: file ?? this.file,
        contentJson: contentJson ?? this.contentJson);
  }
}
